/* @(#)src/sol/s2.5.1/kernel/base/vx_solaris.h	3.18.10.2 14 Jul 1998 05:43:58 -  */
/* #ident "@(#)vxfs:src/sol/s2.5.1/kernel/base/vx_solaris.h	3.18.10.2" */

/*
 * Copyright (c) 1997 VERITAS Software Corporation.  ALL RIGHTS RESERVED.
 * UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT
 * LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE
 * IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION
 * OR DISCLOSURE.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND
 * TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE,
 * OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
 * EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE.
 *
 *		RESTRICTED RIGHTS LEGEND
 * USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS
 * SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH
 * (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND
 * COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013.
 *		VERITAS SOFTWARE
 * 1600 PLYMOUTH STREET, MOUNTAIN VIEW, CA 94043
 */

#ifndef	_FS_VXFS_VX_SOLARIS_H
#define	_FS_VXFS_VX_SOLARIS_H

#define VX_RW_WRITE_HELD(rwlkp)  (((rwlock_impl_t *)rwlkp)->un.rw.holdcnt == -1)
#define VX_RW_LOCK_HELD(rwlkp)	(((rwlock_impl_t *)rwlkp)->un.rw.holdcnt != 0)

/*
 * Defines for page cache interfaces
 */

#define VX_PAGE_IS_WRLOCKED(pp)		se_excl_assert(&(pp)->p_selock)
#define VX_PAGE_IS_RDLOCKED(pp)		se_shared_assert(&(pp)->p_selock)
#define VX_SEGMAP_PAGEUNLOCK(seg, addr, len, rw) \
		((void) segmap_fault(kas.a_hat, (seg), (addr), \
				     (len), F_SOFTUNLOCK, (rw)))

/*
 * Solaris 2.6 now supports VSOCK where as 2.5.1 does not.
 * We will map VX_VSOCK to VNON for 2.5.1
 */

#define VX_VSOCK VNON

/*
 * Local definition for credentials structure.
 */

typedef struct cred	vx_cred_t;
typedef	u_short		uint16_t;
typedef	long int	off64_t;
typedef struct dirent	vx_dirent_t;
typedef struct flock	vx_flock_t;
typedef struct statvfs	vx_statvfs_t;
typedef unsigned long	vxoff_t;

#define	_VXFS_LITTLE_OFFSETS	1234
#define	_VXFS_BIG_OFFSETS	12345678

#define	_VXFS_OFFSET_TYPE	_VXFS_LITTLE_OFFSETS

#define VX_SYS_MAX_OFF(uiop, ioflag)	(vxoff_t)MAXOFF_T

#define VX_PAGE_IS_FREE(pp)	((pp)->p_free != 0)

#define VX_FOFFMAX_CHK(lfflag)	(0)

#define FLARGEFILE	0

/*
 * With small ofsets file sizes are limited to 2^31 - 1 bytes.
 *
 * The file system size is limited to 2^31 - 1 blocks because
 * daddr_t is signed.
 */

#define	VX_MAX_OFF		(((vxoff_t)1 << 31) - 1)
#define	VX_MAX_BLKOFF(fs)	(vxoff_t)((VX_MAX_OFF + 1) >> (fs)->fs_bshift)
#define VX_MAX_BLOCKS(fs)	(long)VX_MAXFS_BLOCKS(fs)

#define	VX_MAXFS_SECTOFF	(daddr_t)(((vxoff_t)1 << 31) - 1)
#define	VX_MAXFS_BLKOFF(fs)	(vxoff_t)(VX_MAXFS_SECTOFF >> \
					  ((fs)->fs_bshift - DEV_BSHIFT))
#define VX_MAXFS_BLOCKS(fs)	(vxoff_t)(VX_MAXFS_BLKOFF(fs))

/*
 * VX_NBDA: Number of Bytes for Dirent Alignment
 */

#define VX_NBDA	(4 - 1)

#define	VX_PAGEIO_INIT()	{					\
	VX_PGIO_SPIN_INIT(&vx_pageio_kmem_lk);				\
	VX_SVAR_INIT(&vx_pageio_freelist_sv, VX_PAGEIO_FREELIST_NAME);	\
}

#define	VX_PAGEIO_DEINIT()	{					\
	VX_SPIN_DEINIT(&vx_bc_bfree_lk);				\
	VX_SVAR_DEINIT(&vx_bc_biomem_sv);				\
	VX_SPIN_DEINIT(&vx_pageio_kmem_lk);				\
	VX_SVAR_DEINIT(&vx_pageio_freelist_sv);				\
}
#ifdef TED_
#define	VX_PAGEIO_MEM_LOCK(iplp)	xted_pageio_mem_lock(iplp)	/*TED_*/
#define	VX_PAGEIO_MEM_UNLOCK(ipl)	xted_pageio_mem_unlock(ipl)	/*TED_*/
#else /*TED_*/
#define	VX_PAGEIO_MEM_LOCK(iplp)					\
	VX_SPIN_LOCK(&vx_pageio_kmem_lk, VX_SPLBASE, iplp)
#define	VX_PAGEIO_MEM_UNLOCK(ipl)					\
	VX_SPIN_UNLOCK(&vx_pageio_kmem_lk, ipl)
#endif /*TED_*/

#define VX_HAT_PAGEUNLOAD(pp)		hat_pageunload(pp)
#define VX_HAT_UNLOCK(hat, addr, len)	hat_unlock(hat, &kas, addr, len)

#define	DIRENTPAD  ((int)((struct dirent *)0)->d_name)
#define VX_LENDIRDOT (sizeof (struct vx_dirdot))
#define VX_LENDIRDOTDOT (sizeof (struct vx_dirdotdot))

/*
 * The maximum number of inodes supported on version 3 file systems
 * and beyond.  Version 1 file systems are limited by their geometry
 * and version 2 by VX_MAX_SMALLOFF.
 *
 * This limit of 2^30 inode is somewhat arbitrary and based on the
 * scalability of our algorithms.  The real limit is probably close
 * to (2^32 - fs_bsize).
 */

#define	VX_MAX_NINODE	(1024 * 1024 * 1024)

typedef struct vx_hyper {
	union vh_hyper {
		unsigned char		vh_bytes[8];
		unsigned long		vh_words[2];
	} vh_hyper;
} vxhyper_t;

#define	vh_low		vh_hyper.vh_words[1]
#define	vh_high		vh_hyper.vh_words[0]
#define	vh_highbyte	vh_hyper.vh_bytes[0]

#define	vh_value	vh_low

#define VX_HYPER_GETOFF(hy)		((hy).vh_low)
#define VX_HYPER_GETTYP(hy)		((hy).vh_highbyte)
#define VX_HYPER_SETOFF(hy, val)	((hy).vh_low= (val))
#define VX_HYPER_SETTYP(hy, val)	((hy).vh_highbyte = (val))

/*
 * Determine if two hypers are equal.
 */

#define VX_HYPER_EQ(h1, h2)						\
	((h1)->vh_high == (h2)->vh_high && (h1)->vh_low == (h2)->vh_low)

/*
 * Determine if a hyper is zero.
 */

#define VX_HYPER_ZERO(h)	((h).vh_high == 0 && (h).vh_low == 0)

/*
 * Non-zero iff h1 < h2.
 */

#define VX_HYPER_LT(h1, h2)						\
	((h1)->vh_high < (h2)->vh_high ||				\
	 ((h1)->vh_high == (h2)->vh_high &&				\
	  (h1)->vh_low < (h2)->vh_low))

/*
 * Non-zero iff h1 <= h2.
 */

#define VX_HYPER_LTE(h1, h2)						\
	((h1)->vh_high < (h2)->vh_high ||				\
	((h1)->vh_high == (h2)->vh_high &&				\
	 (h1)->vh_low <= (h2)->vh_low))

/*
 * h1 = h2 + i.
 */

#define VX_HYPER_ADD(h1, h2, i) {					\
	(h1)->vh_high = (h2)->vh_high;					\
	(h1)->vh_low = (h2)->vh_low + (i);				\
	if ((h1)->vh_low < (h2)->vh_low) {				\
		(h1)->vh_high++;					\
	}								\
}

#define VX_HYPER_INC(h) {						\
	if (++(h)->vh_low == 0) {					\
		(h)->vh_high++;						\
	}								\
}

/*
 * h1 = h2 - i.
 */

#define VX_HYPER_SUB(h1, h2, i) {					\
	(h1)->vh_high = (h2)->vh_high;					\
	(h1)->vh_low = (h2)->vh_low - (i);				\
	if ((h1)->vh_low > (h2)->vh_low) {				\
		(h1)->vh_high--;					\
	}								\
}

/*
 * Convert a hyper to a disk address type.
 */

#define VX_HYPER_TO_DADDR(fs, h, d)					\
	((d) = ((h).vh_low >> (fs)->fs_bshift) +			\
	 ((h).vh_high << (NBBY * sizeof (long) - (fs)->fs_bshift)))

/*
 * Hidden dot and dotdot entries for readdir().
 * These are just instances of the dirent structure with the
 * right name length.
 */

struct vx_dirdot {
	ino_t		d_ino;
	off_t		d_off;
	unsigned short	d_reclen;
	char		d_name[2];
};

struct vx_dirdotdot {
	ino_t		d_ino;
	off_t		d_off;
	unsigned short	d_reclen;
	char		d_name[6];
};

extern struct vx_dirdot		vx_dirdot;
extern struct vx_dirdotdot	vx_dirdotdot;

/*
 * Solaris 2.6 defines a new flag for VOP_GETATTR. Solaris 2.5.1
 * does not support it.
 */

#define VX_ATTR_REAL	0

/*
 * OS version specific function prototypes
 */

/*
 * Functions from vx_swap.c
 */

extern int	vx_pageio(struct vnode *, struct page *, u_int, u_int, int,
			  struct cred *);

/*
 * functions from vx_solaris.c
 */

extern int	vx_create(struct vnode *, char *, struct vattr *,
			  enum vcexcl, int, struct vnode **, struct cred *);
extern int	vx_mkdir(struct vnode *, char *, struct vattr *,
			 struct vnode **, struct cred *);
extern int	vx_putapage(struct vnode *, struct page *, u_int *,
				u_int *, int, vx_cred_t *);
extern int	vx_putinval(struct vnode *, struct page *, u_int *,
				u_int *, int, vx_cred_t *);

#endif /* _FS_VXFS_VX_SOLARIS_H */
