/* @(#)src/sol/s2.6/kernel/base/vx_solaris.h	3.18.10.1 06 Feb 1998 13:08:38 -  */
/* #ident "@(#)vxfs:src/sol/s2.6/kernel/base/vx_solaris.h	3.18.10.1" */

/*
 * Copyright (c) 1997 VERITAS Software Corporation.  ALL RIGHTS RESERVED.
 * UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT
 * LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE
 * IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION
 * OR DISCLOSURE.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND
 * TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE,
 * OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
 * EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE.
 *
 *		RESTRICTED RIGHTS LEGEND
 * USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS
 * SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH
 * (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND
 * COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013.
 *		VERITAS SOFTWARE
 * 1600 PLYMOUTH STREET, MOUNTAIN VIEW, CA 94043
 */

#ifndef	_FS_VXFS_VX_SOLARIS_H
#define	_FS_VXFS_VX_SOLARIS_H

#include <sys/flock.h>

typedef	struct dirent64		vx_dirent_t;
typedef struct flock64		vx_flock_t;
typedef struct statvfs64	vx_statvfs_t;
typedef unsigned long long	vx_u64_t;	/* 64 bit arithmetic (if any) */
typedef long long		vx_s64_t;	/* 64 bit signed math  */
typedef unsigned long long	vxoff_t;

#define	_VXFS_BIG_OFFSETS	12345678
#define	_VXFS_LITTLE_OFFSETS	1234

#define	_VXFS_OFFSET_TYPE	_VXFS_BIG_OFFSETS

#define VA_LIST		va_list
#define VA_START	va_start
#define VA_ARG		va_arg

/*
 * With big offsets, version 3 file systems are limited to file
 * sizes of 2^31 device blocks (sectors), to avoid overflowing the
 * dbd field in the vm code (limit is 2^28 pages).
 *
 * The file system size is limited to 2^31 - 1 blocks because
 * daddr_t is signed.
 */

#define	VX_MAX_OFF		(((vxoff_t)1 << (31 + VX_MINBSHIFT)) - 1)
#define	VX_MAX_BLKOFF(fs)	(vxoff_t)((VX_MAX_OFF + 1) >> (fs)->fs_bshift)
#define VX_MAX_BLOCKS(fs)	(long)VX_MAXFS_BLOCKS(fs)

#define	VX_MAXFS_SECTOFF	(daddr_t)(((vxoff_t)1 << 31) - 1)
#define	VX_MAXFS_BLKOFF(fs)	(vxoff_t)(VX_MAXFS_SECTOFF >> \
					  ((fs)->fs_bshift - DEV_BSHIFT))
#define VX_MAXFS_BLOCKS(fs)	(vxoff_t)(VX_MAXFS_BLKOFF(fs))

#define	VX_SYS_MAX_OFF(uiop, ioflag)	\
	(vxoff_t)((VX_UIO_FMODE(uiop) & FOFFMAX) ? MAXOFFSET_T : MAXOFF_T)

#define VX_PAGE_IS_FREE(pp)	PP_ISFREE(pp)

#define FLARGEFILE	FOFFMAX

/*
 * Solaris 2.6 seems to have done away with "HZ".
 * O/S code uses extern int hz now, but applications can get HZ from param.h
 */

#ifdef _KERNEL
#define HZ	hz
#endif

#define VX_VSOCK	VSOCK

#define VX_FOFFMAX_CHK(lfflag)	((lfflag & FOFFMAX) ? 0 : EOVERFLOW)

/*
 * The maximum number of inodes supported on version 3 file systems
 * and beyond.  Version 1 file systems are limited by their geometry
 * and version 2 by VX_MAX_SMALLOFF.
 *
 * This limit of 2^30 inode is somewhat arbitrary and based on the
 * scalability of our algorithms.  The real limit is probably close
 * to (2^32 - fs_bsize).
 */

#define	VX_MAX_NINODE	(1024 * 1024 * 1024)

/*
 * Set up definitions for accessing an 8 byte data type.
 */

typedef struct vx_hyper {
	union vh_hyper {
		unsigned char		vh_bytes[8];
		unsigned long		vh_words[2];
		unsigned long long	vh_64bits;
	} vh_hyper;
} vxhyper_t;

typedef vxhyper_t	vx_blkcnt64_t;		/* 64 bit count of blocks */

#define	vh_low		vh_hyper.vh_words[1]
#define	vh_high		vh_hyper.vh_words[0]
#define	vh_highbyte	vh_hyper.vh_bytes[0]

#define	vh_value	vh_hyper.vh_64bits

#define VX_HYPER_LOWMASK	0x00ffffffffffffff
#define VX_HYPER_HIGHMASK	0xff00000000000000
#define VX_HYPER_GETOFF(hy)	((hy).vh_hyper.vh_64bits & VX_HYPER_LOWMASK)
#define VX_HYPER_GETTYP(hy)	((hy).vh_highbyte)
#define VX_HYPER_SETTYP(hy, val)					\
	((hy).vh_highbyte = (val))
#define VX_HYPER_SETOFF(hy, val)					\
	(hy).vh_hyper.vh_64bits = (((hy).vh_hyper.vh_64bits &		\
				     VX_HYPER_HIGHMASK) |		\
				    ((unsigned long long)(val) &	\
				     VX_HYPER_LOWMASK))

/*
 * Determine if two hypers are equal.
 */

#define VX_HYPER_EQ(h1, h2)	((h1)->vh_value == (h2)->vh_value)

/*
 * Determine if a hyper is zero.
 */

#define VX_HYPER_ZERO(h)	((h).vh_value == (unsigned long long)0)

/*
 * Non-zero iff h1 < h2.
 */

#define VX_HYPER_LT(h1, h2)	((h1)->vh_value < (h2)->vh_value)

/*
 * Non-zero iff h1 <= h2.
 */

#define VX_HYPER_LTE(h1, h2)	((h1)->vh_value <= (h2)->vh_value)

/*
 * Math operations on hypers
 */

#define VX_HYPER_ADD(h1, h2, i)	(h1)->vh_value = (h2)->vh_value + (i)
#define VX_HYPER_SUB(h1, h2, i)	(h1)->vh_value = (h2)->vh_value - (i)
#define VX_HYPER_INC(h)		(++(h)->vh_value)
#define VX_HYPER_DEC(h)		(--(h)->vh_value)

/*
 * Convert a hyper to a disk address type.
 */

#define VX_HYPER_TO_DADDR(fs, h, d)					\
	((d) = (h).vh_value >> (fs)->fs_bshift)

/*
 * macros for rwlocks
 */

#define VX_RW_WRITE_HELD(rwlkp)	rw_write_held(rwlkp)
#define VX_RW_LOCK_HELD(rwlkp)	rw_lock_held(rwlkp)

/*
 * Defines for page cache interfaces
 */

#define VX_PAGE_IS_WRLOCKED(pp)		PAGE_LOCKED_SE(pp, SE_EXCL)
#define VX_PAGE_IS_RDLOCKED(pp)		PAGE_LOCKED_SE(pp, SE_SHARED)
#define VX_SEGMAP_PAGEUNLOCK(seg, addr, len, rw) \
		((void) segmap_fault(kas.a_hat, (seg), (addr), \
                                     (len), F_SOFTUNLOCK, (rw)))

/*
 * VX_NBDA: Number of Bytes for Dirent Alignment
 */

#define VX_NBDA	(8 - 1)

#define VX_PAGEIO_INIT()
#define VX_PAGEIO_DEINIT()

#define VX_HAT_PAGEUNLOAD(pp)	hat_pageunload(pp, HAT_FORCE_PGUNLOAD)
#define VX_HAT_UNLOCK(hat, addr, len)	hat_unlock(hat, addr, len)

struct vx_dirdot {
    ino64_t		d_ino;
    off64_t		d_off;
    unsigned short	d_reclen;
    char		d_name[6];
};

struct vx_dirdotdot {
    ino64_t		d_ino;
    off64_t		d_off;
    unsigned short	d_reclen;
    char		d_name[6];
};

extern struct vx_dirdot		vx_dirdot;
extern struct vx_dirdotdot	vx_dirdotdot;

#define VX_LENDIRDOT (sizeof (struct vx_dirdot))
#define VX_LENDIRDOTDOT (sizeof (struct vx_dirdotdot))
#define DIRENTPAD  ((int)((vx_dirent_t *)0)->d_name)

/*
 * VOP_GETATTR on Solaris 2.6 gets an additional flag called as
 * ATTR_REAL.
 */

#define VX_ATTR_REAL	ATTR_REAL

/*
 * OS version specific function prototypes
 */

typedef struct cred	vx_cred_t;

/*
 * Functions from vx_swap.c
 */

extern int	vx_pageio(struct vnode *, struct page *, u_offset_t, u_int, int,
			  struct cred *);

/*
 * functions from vx_solaris.c
 */

extern int	vx_create(struct vnode *, char *, struct vattr *,
			  enum vcexcl, int, struct vnode **, struct cred *,
			  int);
extern int	vx_mkdir(struct vnode *, char *, struct vattr *,
			 struct vnode **, struct cred *);

extern int	vx_putinval(struct vnode *, struct page *, u_offset_t *,
				u_int *, int, vx_cred_t *);
extern int	vx_putapage(struct vnode *, struct page *, u_offset_t *,
				u_int *, int, vx_cred_t *);

typedef (*vx_creatfptr_t)(struct vnode *, char *, struct vattr *, vcexcl_t,
			  int, struct vnode **, vx_cred_t *, int);

#endif /* _FS_VXFS_VX_SOLARIS_H */
