/* @(#)src/common/kernel/base/vx_inode.h	3.66 04 Nov 1997 16:06:03 -  */
/* #ident "@(#)vxfs:src/common/kernel/base/vx_inode.h	3.66" */

/*
 * Copyright (c) 1997 VERITAS Software Corporation.  ALL RIGHTS RESERVED.
 * UNPUBLISHED -- RIGHTS RESERVED UNDER THE COPYRIGHT
 * LAWS OF THE UNITED STATES.  USE OF A COPYRIGHT NOTICE
 * IS PRECAUTIONARY ONLY AND DOES NOT IMPLY PUBLICATION
 * OR DISCLOSURE.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND
 * TRADE SECRETS OF VERITAS SOFTWARE.  USE, DISCLOSURE,
 * OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
 * EXPRESS WRITTEN PERMISSION OF VERITAS SOFTWARE.
 *
 * 		RESTRICTED RIGHTS LEGEND
 * USE, DUPLICATION, OR DISCLOSURE BY THE GOVERNMENT IS
 * SUBJECT TO RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH
 * (C) (1) (ii) OF THE RIGHTS IN TECHNICAL DATA AND
 * COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013.
 * 		VERITAS SOFTWARE
 * 1600 PLYMOUTH STREET, MOUNTAIN VIEW, CA 94043
 */

#ifndef	_FS_VXFS_VX_INODE_H
#define	_FS_VXFS_VX_INODE_H

/*
 * The head of the inode cache lists.
 *
 * The offset of i_cforw and i_cback must match struct vx_inode.
 */

struct vx_icache {
	struct vx_inode	*i_filler[4];	/* filler for hash chains */
	struct vx_inode	*i_cforw;	/* icache forw pointer */
	struct vx_inode	*i_cback;	/* icache back pointer */
	vx_spin_t	i_list_lk;	/* lock for this list */
	int		i_count;	/* count of inodes on list */
};

/*
 * The head of the various inode free/dirty lists.
 *
 * The offset of av_forw, av_back, i_dxw_forw, and i_dxw_back must
 * match struct vx_inode.
 */

struct vx_ifreehead {			/* must match struct vx_inode */
	struct vx_inode *i_filler[6];	/* 0x00 filler for hash chains */
	struct vx_inode *av_forw;	/* 0x10 freelist chain */
	struct vx_inode *av_back;	/* 0x14 " */
	struct vx_inode *i_dxw_forw;	/* 0x18 delayed extending write list */
	struct vx_inode *i_dxw_back;	/* 0x1c " */
	int		i_count;	/* 0x20 inodes on list */
	int		i_gencount;	/* 0x24 list generation count */
	vx_spin_t	i_freelist_lk;	/* 0x28 lock freelist and ident hash */
	struct vx_ifreehead *i_next;	/* 0x2c chain of all freelists */
};

/*
 * Inode list for inodes under reorg. Used during replay of extended operations.
 */

struct vx_ireorglist {
	struct vx_inode		*i_reorgip;
	struct vx_ireorglist	*i_next;
};

/*
 * The inode lock hash chain headers.
 */

struct vx_hilock {
	struct vx_ilock	*l_forw;	/* next lock in hash chain */
	struct vx_ilock	*l_back;	/* previous lock in hash chain */
	vx_spin_t	*l_lhash_lkp;	/* lock for this hash chain */
};

/*
 * The inode hash chain headers.
 */

struct vx_hinode {		/* must match struct vx_inode */
	struct vx_inode	*i_forw;
	struct vx_inode	*i_back;
	struct vx_inode	*i_bforw;
	struct vx_inode	*i_bback;
	struct vx_ifreehead *i_identlock; /* lock for ident hash chain */
	vx_spin_t	*i_bhash_lkp;	  /* lock for block hash chain */
};

/*
 * I/O statistics for (device) files.
 */

struct vx_iostat {
	int	nreads;			/* Number of reads */
	int	nblksread;		/* Number of blocks read */
	int	nwrites;		/* Number of Writes */
	int	nblkswrite;		/* Number of blocks written */
	clock_t	readtime;		/* Total time required for nreads */
	clock_t	writetime;		/* Total time required for nwrites */
};

#define i_nreads	i_iostat.nreads
#define i_nwrites	i_iostat.nwrites
#define i_nblksread	i_iostat.nblksread
#define i_nblkswrite	i_iostat.nblkswrite
#define i_readtime	i_iostat.readtime
#define i_writetime	i_iostat.writetime

/*
 * The extent structure.
 */

struct vx_extent1 {
	vxoff_t	ex1_off;	/* offset of extent in file */
	vxoff_t	ex1_sz;		/* size of extent in bytes */
	daddr_t ex1_st;		/* start block of extent */
	u_int	ex1_flags;	/* flags */
	ulong	ex1_elen;	/* fixed extent size if a multi extent run */
	long	ex1_dev;	/* devid of extent */
	struct vx_inode	*ex1_ip;/* inode were extent was found */
};

struct vx_extent2 {
	vxoff_t ex2_lbno;	/* logical start block buffer within file */
	ulong	ex2_boff;	/* offset within buffer */
	ulong	ex2_bsz;	/* size of buffer */
	daddr_t ex2_bst;	/* start block of buffer */
};

struct vx_extent {
	struct vx_extent1	ex1;
	struct vx_extent2	ex2;
};

/*
 * flags for ex_flags
 */

#define	VX_EXF_NEW		0x0001	/* extent is newly allocated */
#define	VX_EXF_EXTEND		0x0002	/* OK to extend previous extent */

#define	ex_off		ex1.ex1_off
#define	ex_sz		ex1.ex1_sz
#define	ex_st		ex1.ex1_st
#define	ex_flags	ex1.ex1_flags
#define	ex_elen		ex1.ex1_elen
#define	ex_dev		ex1.ex1_dev
#define	ex_ip		ex1.ex1_ip
#define	ex_boff		ex2.ex2_boff
#define	ex_bsz		ex2.ex2_bsz
#define	ex_bst		ex2.ex2_bst
#define	ex_lbno		ex2.ex2_lbno

/*
 * The incore inode structure.
 *
 * Note that the embedded vxoff_t are accounted as 64-bit entities
 * in the structure offsets in the comments.
 */

struct vx_inode {
	struct vx_inode *i_forw;   /* 0x00 inode hash chain */
	struct vx_inode *i_back;   /* 0x04 inode hash chain */
	struct vx_inode *i_bforw;  /* 0x08 inode block hash chain */
	struct vx_inode *i_bback;  /* 0x0c inode block hash chain */
	struct vx_inode *i_cforw;  /* 0x10 inode cache chain */
	struct vx_inode *i_cback;  /* 0x14 inode cache chain */
	struct vx_inode *av_forw;  /* 0x18 freelist chain */
	struct vx_inode *av_back;  /* 0x1c freelist chain */
	struct vx_inode *i_dxw_forw; /* 0x20 delayed extending write list */
	struct vx_inode *i_dxw_back; /* 0x24 delayed extending write list */
	struct vx_mlinkhd i_blink; /* 0x28 head of buffer mlinks */
	struct vx_mlinkhd i_dlink; /* 0x30 head of logged write mlinks */
	struct vx_mlinkhd i_mlink; /* 0x38 head of inode modification mlinks */
	struct vx_mlinkhd i_alink; /* 0x40 async iupdate mlinks */
	struct vx_inode *i_aupd;   /* 0x48 async iupdate linked list */
	u_int	i_flag;		/* 0x4c flags - see below */
	u_int	i_sflag;	/* 0x50 flags protected by inode spin lock */
	u_int	i_intrflag;	/* 0x54 flags touched at interrupt time */
	vxoff_t	i_nsize;	/* 0x58 "new" bytes in file for IDELXWRI */
	vxoff_t	i_wsize;	/* 0x60 size of file if write suceeds */
	vxoff_t	i_dsize;	/* 0x68 max possible inode size on disk */
	dev_t	i_dev;		/* 0x70 device where inode resides */
	ino_t	i_number;	/* 0x74 inode number */
	struct vx_fset *i_fset;	/* 0x78 fileset associated with this inode */
	struct vx_fs   *i_fs;	/* 0x7c aggregate associated with this inode */
	struct vx_ias  *i_ias;	/* 0x80 inode address space pointer */
	struct vx_fsetid *i_fsi;  /* 0x84 fileset identity struct ptr */
	int	i_advise;	/* advisories */
	vxoff_t	i_ndblocks;	/* number of blocks in direct extents */
	vxoff_t	i_diroff;	/* offset in dir, where last entry found */
	ino_t	i_lastino;	/* last inode allocated into directory */
	int	i_dirstamp;	/* timestamp from dir search */
	struct vx_extent1 i_ext;/* last extent for bmap cache */
	int	i_tranid;	/* tran id for logged write flushing */
	int	i_max_lwrid;	/* tran id of last logged write tran */
	vxoff_t	i_nextr;	/* expected offset of next read */
	vxoff_t	i_nextio;	/* offset of next read ahead I/O point */
	vxoff_t	i_raend;	/* offset where read ahead area ends */
	vxoff_t	i_nextw;	/* expected offset of next write */
	vxoff_t	i_wflush;	/* offset of last write flush point */
	vxoff_t	i_wseqstart;	/* start of series of sequential writes */
	vxoff_t	i_lastflushoff;	/* offset of last completed write flush */
	vxoff_t	i_nextfault;	/* next fault expected on vnode */
	vxoff_t	i_getraoff;	/* getpage read ahead offset */
	size_t	i_ralen;	/* length of read ahead area */
	short	i_nodealloc_fsq;/* block inode dealloc fsq lock */
	short	i_nodealloc_idxw; /* block inode dealloc delxwri lock */
	short	i_nodealloc_icache; /* block inode dealloc icache lock */
	short	i_inreuse;	/* block vx_itryhold references */
	daddr_t	*i_swapmap;	/* block map for a swap file */
	short	i_readcnt;	/* count of procs reading the file */
	short	i_swmap;	/* file may have shared writable mappings */
	long	i_swapcnt;	/* number times used as swap file */
	long	i_mapcnt;	/* count of procs mapping the file */
	long	i_afree;	/* free space in attribute area */
	caddr_t	i_attr;		/* pointer to attribute area */
	long	i_aclcnt;	/* acl count */
	long	i_daclcnt;	/* default acl count */
	long	i_iaclcnt;	/* count of acls to inherit */
	long	i_acllink;	/* flag for linking directory acls */
	long	i_modemask;	/* inheritance permission mask */
	long	i_attrlen;	/* length of attribute area */
	daddr_t	i_bno;		/* ilist block - in sectors */
	int	i_boff;		/* offset of disk inode in ilist buffer */
	int	i_blen;		/* length of ilist buffer */
	struct vx_icache *i_icache; /* icache chain */
	struct vx_ifreehead *i_freelist; /* freelist inode currently on */
	struct vx_ifreehead *i_ilist; /* inode freelist/ident hash lock */
	struct vx_hinode    *i_bhash; /* block hash chain head */
	struct vx_hilock    *i_lhash; /* lock hash chain head */
	long	i_pageflushes;	/* blocks of outstanding write flush */
	clock_t	i_ftime;	/* time inode put on freelist */
	clock_t	i_sizetime;	/* time of last size update for delxwri */
	int	i_trangen;	/* generation count of transactions */
	int	i_agen;		/* access generation count */
	ulong	i_vcode;	/* version code attribute */
	struct vx_dquot *i_dquot[VX_MAXQUOTAS]; /* quota information */
	struct vnode *i_vnode;	/* vnode associated with this inode */
	struct vx_inode	*i_push;/* inode push chain */
	struct vx_bmapops *i_bops;  /* bmap ops */
	int	i_dxwgencnt;	    /* saved gen count for dxw list */
	struct	vx_icommon i_ic;    /* disk inode */
	vx_spin_t i_intrlock;		/* interrupt lock */
	vx_spin_t i_spinlock;		/* inode spin lock */
	vx_sleep_t *i_ilock;		/* inode lock */
	vx_rwsleep_t *i_plock;		/* putpage lock */
	vx_rwsleep_t *i_bmaplock;	/* bmap lock */
	struct vx_ilock	*i_slocks;	/* shared inode locks */
	u_long			i_hsm_change;		/* HSM_ */
	struct kdm_vnode	i_hsm_kvnode;		/* HSM_ */
	caddr_t			i_hsm_attr;		/* HSM_ */
	int			i_hsm_attrlen;		/* HSM_ */
	int			i_ueread;   /* unexpected read count */
	long			i_lmode;    /* lock mode */
	struct vx_iostat	i_iostat;   /* i/o statistics for fdd */
};					    /* XXX is length */

#define VX_KDM_ITOV(ip)		((struct kdm_vnode *)&(ip)->i_hsm_kvnode)
#define VX_KDM_VTOI(kp)		((struct vx_inode *)(kp)->kdv_data)

/*
 * Aliases for the disk inode fields.
 */

#define	i_mode		i_ic.ic_mode
#define	i_nlink		i_ic.ic_nlink
#define	i_uid		i_ic.ic_uid
#define	i_gid		i_ic.ic_gid
#define	i_size		i_ic.ic_size.vh_value

#define	i_atime		i_ic.ic_atime
#define	i_mtime		i_ic.ic_mtime
#define	i_ctime		i_ic.ic_ctime

#define	i_aflags	i_ic.ic_aflags
#define	i_orgtype	i_ic.ic_orgtype
#define	i_eopflags	i_ic.ic_eopflags
#define	i_eopdata	i_ic.ic_eopdata

#define	i_rdev		i_ic.ic_ftarea.ic_ftarea_dev.ic_rdev
#define	i_dotdot	i_ic.ic_ftarea.ic_ftarea_dir.ic_dotdot
#define	i_dflags	i_ic.ic_ftarea.ic_ftarea_dir.ic_dflags
#define	i_fixextsize	i_ic.ic_ftarea.ic_ftarea_reg.ic_fixextsize
#define	i_reserve	i_ic.ic_ftarea.ic_ftarea_reg.ic_reserve
#define	i_matchino	i_ic.ic_ftarea.ic_ftarea_vxspec.ic_matchino
#define	i_fsetindex	i_ic.ic_ftarea.ic_ftarea_vxspec.ic_fsetindex

#define	i_blocks	i_ic.ic_blocks
#define	i_gen		i_ic.ic_gen
#define	i_lversion	i_ic.ic_vversion.vh_low
#define	i_hversion	i_ic.ic_vversion.vh_high
#define	i_vversion	i_ic.ic_vversion

#define	i_immed		i_ic.ic_org.ic_vx_immed.ic_immed

#define	i_ies		i_ic.ic_org.ic_vx_e4.ic_ies
#define	i_ie		i_ic.ic_org.ic_vx_e4.ic_ie
#define	i_dext		i_ic.ic_org.ic_vx_e4.ic_dext

#define	i_td4		i_ic.ic_org.ic_td4
#define	i_td8		i_ic.ic_org.ic_td8

#define	i_iattrino	i_ic.ic_iattrino

/*
 * flags in i_flag
 */

#define	IUPD		0x00000001	/* file has been modified */
#define	IACC		0x00000002	/* inode access time to be updated */
#define	IMOD		0x00000004	/* inode has been modified */
#define	ICHG		0x00000008	/* inode has been changed */
#define	IATIMEMOD	0x00000010	/* atime modified */
#define	IMTIMEMOD	0x00000020	/* mtime modified */
#define	ITRANLAZYMOD	0x00000040	/* transaction lazy mod */
#define INOCLONE	0x00000080	/* inode does not need push inode */
#define	ISYNCWRITES	0x00000100	/* file has had O_SYNC writes */
#define	IINTRANS	0x00000200	/* inode is in a transaction */
#define	IDELBUF		0x00000400	/* inode has pending delayed buffers */
#define	IGHOST		0x00000800	/* inode is a ghost attribute */
#define	ISHRDATA	0x00001000	/* don't delete data extents */
#define	INOTIMEMOD	0x00002000	/* no time update on invis i/o */
#define	ILOGWRITELOCK	0x00004000	/* inode in logged write transaction */
#define IBADHOLD	0x00008000	/* bad inode but can't idrop */

/*
 * flags in i_intrflag
 */

#define	IDELXWRI	0x0001		/* inode has delxwri data */
#define	ILOGWRITE	0x0002		/* inode has pending logged writes */
#define	IDELAYEDERR	0x0004		/* delayed update error occurred */
#define	IASYNCUPD	0x0008		/* async inode write in progress */

/*
 * flags in i_sflag
 */

#define	IADDRVALID	0x00000001	/* bmap has validated address map */
#define	IBAD		0x00000002	/* inode is bad */
#define	IUEREAD		0x00000004	/* getpage did an unexpected read */
#define	INOBMAPCACHE	0x00000008	/* bmap cache is disabled */
#define	IBADUPD		0x00000010	/* must be marked bad to disk */
#define	IPUTTIME	0x00000020	/* B_FORCE putpage mtime update */
#define	IATTRREM	0x00000040	/* remove attribute on inactive */
#define	INOBMAPCLUSTER	0x00000080	/* prevent bmap from clustering */
#define	IDEVACC		0x00000100	/* update access time on dev/fifo */
#define	IDEVUPDCHG	0x00000200	/* update mod/chg time on dev/fifo */
#define	IXWRITE		0x00000400	/* file has been written flag */
#define	IWRITEMAP	0x00000800	/* file has writable mappings */
#define	IDEVICE		0x00001000	/* file being accessed as device */
#define	ILASTPAGEZEROED	0x00002000	/* last page has been zeroed */

/*
 * Misc clone flags
 */

#define	VX_EOVERLAY	0x0001		/* Setup the next overlay inode */
#define	VX_EPUSH	0x0002		/* Setup the push inode */

/*
 * Iget flags
 */

#define	VX_IG_AFLAG	0x0001		/* Set full fsck flag indication */

/*
 * Iaccess flags
 */

#define	VX_DIRINSERT	1		/* Inserting a directory entry */
#define	VX_DIRDELETE	2		/* Deleting a directory entry */

/*
 * File types for which BSD quotas may be enforced.
 */

#define	VX_BSD_QUOT	(IFIFO | IFCHR | IFBLK | IFDIR | IFREG | IFLNK | \
			 IFSOC | IFATT | IFIAT)

/*
 * Convert between inode pointers and vnode pointers
 */

#define	VX_VTOI(vp)	((struct vx_inode *)(void *)(vp)->v_data)
#define	VX_ITOV(ip)	((ip)->i_vnode)

/*
 * Convert from a vnode or inode to an external file system struct.
 */

#define VX_VTOEXT(vp)	(VX_VFS((vp)->v_vfsp)->vfs_fsext)
#define VX_ITOEXT(ip)	VX_VTOEXT(VX_ITOV(ip))

/*
 * Force any delayed writes on the inode to complete.
 */

#define	VX_DELBUF_FLUSH(ip) {					\
	if ((ip)->i_flag & IDELBUF) {				\
		vx_delbuf_flush(ip);				\
	}							\
}

#define	VX_LOGWRI_FLUSH(ip) {					\
	if ((ip)->i_intrflag & ILOGWRITE) {			\
		vx_logwrite_flush(ip);				\
	}							\
}

#define	VX_DELXWRI_FLUSH(ip) {					\
	if ((ip)->i_intrflag & IDELXWRI) {			\
		vx_idelxwri_flush((ip), 0);			\
	}							\
}

#define	VX_DELXWRI_FLUSH_ASYNC(ip) {				\
	if ((ip)->i_intrflag & IDELXWRI) {			\
		vx_idelxwri_flush((ip), B_ASYNC);		\
	}							\
}

/*
 * Find advisories in effect for current process
 */

#define	VX_ADVISEGET(ip)	((ip)->i_advise)

/*
 * Clean up advisories on close
 *
 * When this macro is invoked in vx_inactive(), the reference count
 * should be 1 (invoked by vx_inactive() as the last reference is
 * being released) or 0 (invoked by vx_idrop()).
 */

#define	VX_ADVISECLOSE(ip) {					\
	XTED_ADVISECLOSE(ip);					\
	(ip)->i_advise = 0;					\
}

/*
 * Put an inode on the cache list.
 */

#define	VX_IPCACHE(icp, ip) {					\
	(icp)->i_cback->i_cforw = (ip);				\
	(ip)->i_cforw = (struct vx_inode *)(icp);		\
	(ip)->i_cback = (icp)->i_cback;				\
	(icp)->i_cback = (ip);					\
	(icp)->i_count++;					\
	(ip)->i_icache = (icp);					\
}

/*
 * Remove an inode from the cache list.
 */

#define	VX_IUNCACHE(ip) {					\
	(ip)->i_cback->i_cforw = (ip)->i_cforw;			\
	(ip)->i_cforw->i_cback = (ip)->i_cback;			\
	(ip)->i_cforw = 0;					\
	(ip)->i_cback = 0;					\
	(ip)->i_icache->i_count--;				\
	(ip)->i_icache = NULL;					\
}

/*
 * Put an inode on the end of the free list.
 */

#define	VX_IPFREE(ip) {						\
	XTED_IPFREE(ip);					\
	(ip)->i_ftime = vx_lbolt;				\
	(ip)->i_freelist->av_back->av_forw = (ip);		\
	(ip)->av_forw = (struct vx_inode *)(ip)->i_freelist;	\
	(ip)->av_back = (ip)->i_freelist->av_back;		\
	(ip)->i_freelist->av_back = (ip);			\
	(ip)->i_freelist->i_count++;				\
	VX_INFO_ADD(vxi_inoinuse, -1);				\
}

/*
 * Put an inode on the front of the free list.  Set i_ftime to be
 * the older of vx_iclean_timelag + 1 seconds ago or the i_ftime
 * of the inode currently at the head of the list.
 */

#define	VX_IPFFREE(ip) {					\
	XTED_IPFREE(ip);					\
	(ip)->i_ftime = (((ip)->i_freelist->av_forw ==		\
			 (struct vx_inode *)(ip)->i_freelist) ||\
			 ((ip)->i_freelist->av_forw)->i_ftime >	\
			 (vx_lbolt - vx_iclean_timelag - 1)) ?	\
			 (vx_lbolt - vx_iclean_timelag - 1) :	\
			 ((ip)->i_freelist->av_forw)->i_ftime;	\
	(ip)->i_freelist->av_forw->av_back = (ip);		\
	(ip)->av_back = (struct vx_inode *)(ip)->i_freelist;	\
	(ip)->av_forw = (ip)->i_freelist->av_forw;		\
	(ip)->i_freelist->av_forw = (ip);			\
	(ip)->i_freelist->i_count++;				\
	VX_INFO_ADD(vxi_inoinuse, -1);				\
}

/*
 * Remove an inode from the freelist.
 */

#define	VX_IUNFREE(ip) {					\
	XTED_IUNFREE(ip);					\
	(ip)->av_back->av_forw = (ip)->av_forw;			\
	(ip)->av_forw->av_back = (ip)->av_back;			\
	(ip)->av_forw = 0;					\
	(ip)->av_back = 0;					\
	(ip)->i_freelist->i_count--;				\
	(ip)->i_freelist = NULL;				\
	(ip)->i_ftime = 0;					\
	VX_INFO_ADD(vxi_inoinuse, 1);				\
}

/*
 * Put an inode on a hash list.
 */

#define	VX_IPHASH(hip, ip) {					\
	(hip)->i_forw->i_back = (ip);				\
	(ip)->i_forw = (hip)->i_forw;				\
	(hip)->i_forw = (ip);					\
	(ip)->i_back = (struct vx_inode *)(hip);		\
}

/*
 * Remove an inode from its hash list.  To allow multiple calls,
 * the forward and back hash pointers point back to the inode.
 */

#define	VX_IUNHASH(ip) {					\
	(ip)->i_back->i_forw = (ip)->i_forw;			\
	(ip)->i_forw->i_back = (ip)->i_back;			\
	(ip)->i_forw = (ip);					\
	(ip)->i_back = (ip);					\
}

/*
 * Put an inode on a block hash list.
 */

#define	VX_IPBHASH(hip, ip) {					\
	(hip)->i_bforw->i_bback = (ip);				\
	(ip)->i_bforw = (hip)->i_bforw;				\
	(hip)->i_bforw = (ip);					\
	(ip)->i_bback = (struct vx_inode *)(hip);		\
}

/*
 * Remove an inode from its block hash list.  To allow multiple calls,
 * the forward and back hash pointers point back to the inode.
 */

#define	VX_IUNBHASH(ip) {					\
	(ip)->i_bback->i_bforw = (ip)->i_bforw;			\
	(ip)->i_bforw->i_bback = (ip)->i_bback;			\
	(ip)->i_bforw = (ip);					\
	(ip)->i_bback = (ip);					\
}

/*
 * Put an inode lock on a hash list.
 */

#define	VX_LPHASH(hlp, lp) {					\
	(hlp)->l_forw->l_back = (lp);				\
	(lp)->l_forw = (hlp)->l_forw;				\
	(hlp)->l_forw = (lp);					\
	(lp)->l_back = (struct vx_ilock *)(hlp);		\
}

/*
 * Remove an inode lock from its hash list.
 */

#define	VX_LRHASH(lp) {						\
	(lp)->l_back->l_forw = (lp)->l_forw;			\
	(lp)->l_forw->l_back = (lp)->l_back;			\
	(lp)->l_forw = (lp);					\
	(lp)->l_back = (lp);					\
}

/*
 * Hash an inode.
 */

#define	VX_HASH(seed, key)	((seed) + (key) * 523)

#define	VX_IHASH(seed, key)	(vx_hinode + \
				 (VX_HASH((seed), (key)) & (vx_nhino - 1)))

#define	VX_LHASH(seed, key)	(vx_hilock + \
				 (VX_HASH((seed), (key)) & (vx_nhino - 1)))

union vx_ipargs {
	struct vx_ipchkver {
		vxhyper_t		version;
	} ipchkser;
	struct vx_ipquotacheck {
		struct vx_fset		*fset;
		int			type;
		ulong			*newidlist;
		int			newidcount;
		int			newidlimit;
		int			dqheld;
	} ipquotacheck;
};

/*
 * This overlays the fid structure (see mount.h).
 */

struct vx_vfid {
	u_short	vfid_len;
	ino_t	vfid_ino;
	long	vfid_gen;
};

#define	MAKEIMODE(x, y) ((vx_vttoif(x)) | ((y) & (IFMT | 0xfff)))

/*
 * Structure used by resize to keep track of changes to structural inodes.
 */

struct vx_rsi {
	struct vx_inode	*rsi_ip;
	vxoff_t		rsi_osize;
	vxoff_t		rsi_nsize;
};

/*
 * The attribute code that modifies attributes, particular the attribute
 * inheritance code, requires a number of inode locks.  Some must be
 * appropriately cleaned up/reacquired when a transaction commits and
 * all must be released when the operation completes (an operation may
 * require multiple transactions).
 *
 * See vx_attr.c for a description of attribute locking.  The fields
 * are arranged in the order that locks are acquired.  All non_NULL
 * fields represent inodes that are held and locked.  The locks are:
 *
 * Field	Lock	Explanation
 *
 * vt_dirinip	rwlock	indirect attribute inode for "parent" inode
 * vt_preinip	rwlock	pre-existing indirect attribute inode for target
 * vt_preaip	rwlock	pre-existing attribute inode for target
 * vt_newaip	ilock	newly allocated (or modified) attribute inode
 *			for target
 * vt_newinip	ilock	newly allocated (or modified) indirect attribute
 *			inode for target
 * vt_oldaip	none	an old attribute inode removed in the transaction
 *
 * vt_preinip and vt_newinip may refer to the same inode, if it was
 * pre-existing and modified in the transaction.  vt_preaip and
 * vt_newaip may also refer to the same inode, typically when the
 * inode requires a multi-transaction allocation.
 */

struct vx_attrlocks {
	struct vx_inode *vt_dirinip;	/* directory (parent) indirect inode */
	struct vx_inode *vt_preinip;	/* pre-existing indirect inode */
	struct vx_inode	*vt_preaip;	/* pre-existing attribute inode */
	struct vx_inode	*vt_newaip;	/* new attribute inode */
	struct vx_inode	*vt_newinip;	/* new indirect attribute inode */
	struct vx_inode *vt_oldaip;	/* old (removed) attribute inode */
};

/*
 * Parameters passed through dircreate for inheritance.
 */

struct vx_inherit {
	struct vx_attrlocks	vh_alocks;	/* 0x0 attribute locks */
	struct vx_inode		*vh_linklist[5]; /* 0x18 attr inodes linked */
	int			vh_nlink;	/* 0x2c num of entries */
	struct vx_tran		*vh_tranp;	/* 0x30 current transaction */
	int			vh_onetran;	/* 0x34 fast inheritance */
	uid_t			vh_uid;		/* 0x38 uid to be allocated */
	gid_t			vh_gid;		/* 0x3c gid to be allocated */
	int			vh_tranbytes;	/* 0x40 tranbytes needed */
};

#define	VX_PUBLIC_CLASS	1000	/* classes below 1000 are reserved */

#define	VX_ATTR_IGNORE	1	/* Don't allow operation to see attribute */
#define	VX_ATTR_KEEP	2	/* Perform operation as requested */
#define	VX_ATTR_NEW	3	/* Substitute new attribute data */
#define	VX_ATTR_LINK	4	/* Link to parent attribute */

struct vx_attrop {
	int	(*aop_listop)();	/* Listing/Reading attributes */
	int	(*aop_changeop)();	/* Setting/Removing attributes */
	int	(*aop_creatop)();	/* Inheriting attributes */
	int	(*aop_newop)();		/* New attributes on file create */
};

struct vx_attrreg {
	struct vx_attrreg	*ar_next;	/* next registered class */
	long			ar_class;	/* registered class */
	struct vx_attrop	ar_aops;	/* attribute operations */
	struct vx_attrreg	*ar_newop_next;	/* next class with newop */
};

#define	VX_NREGHASH	64	/* num of hash chains of registered classes */

/*
 * Structure to control multi-threaded processing of flushing.
 */

struct vx_flushargs {
	struct vx_flushargs	*fa_pnext;	/* next pointer */
	vx_event_t		fa_done_ev;	/* done event */
	int			fa_result;	/* error code */
	void			*fa_private1;	/* first arg to function */
};

/*
 * Structure to control multi-threaded inode cache traversals.
 */

struct vx_icacheargs {
	struct vx_icacheargs	*ica_pnext;	/* next pointer */
	struct vx_icache	*ica_icp;	/* icache list to process */
	vx_event_t		ica_done_ev;	/* done event */
	int			ica_result;	/* error code */
	void			*ica_private1;	/* first arg to function */
	void			*ica_private2;	/* second arg to function */
	void			*ica_private3;	/* third arg to function */
	void			*ica_private4;	/* fourth arg to function */
};

/*
 * This structure controls work items for threads.  A work thread should
 * take an item off the worklist queue and process it.
 */

struct vx_worklist {
	struct vx_worklist	*wl_next;		/* next work item */
	struct vx_worklist	*wl_prev;		/* prev work item */
	void			(*wl_workfunc)();	/* function to call */
	void			*wl_args;		/* args to function */
	int			wl_tag;			/* ID tag */
};

#endif	/* _FS_VXFS_VX_INODE_H */
